@@ -526,6 +526,9 @@ netsnmp_subtree_load(netsnmp_subtree *new_sub, const char *context_name)
     return 0;
 }
 
+/*
+ * Note: reginfo will be freed on failures
+ */
 int
 netsnmp_register_mib(const char *moduleName,
                      struct variable *var,
@@ -548,8 +551,15 @@ netsnmp_register_mib(const char *moduleName,
     struct register_parameters reg_parms;
     int old_lookup_cache_val = netsnmp_get_lookup_cache_size();
 
+    if (moduleName == NULL ||
+        mibloc     == NULL) {
+        /* Shouldn't happen ??? */
+        netsnmp_handler_registration_free(reginfo);
+        return MIB_REGISTRATION_FAILED;
+    }
     subtree = (netsnmp_subtree *)calloc(1, sizeof(netsnmp_subtree));
     if (subtree == NULL) {
+        netsnmp_handler_registration_free(reginfo);
         return MIB_REGISTRATION_FAILED;
     }
 
@@ -560,6 +570,7 @@ netsnmp_register_mib(const char *moduleName,
 
     /*  Create the new subtree node being registered.  */
 
+    subtree->reginfo = reginfo;
     subtree->name_a  = snmp_duplicate_objid(mibloc, mibloclen);
     subtree->start_a = snmp_duplicate_objid(mibloc, mibloclen);
     subtree->end_a   = snmp_duplicate_objid(mibloc, mibloclen);
@@ -589,7 +600,6 @@ netsnmp_register_mib(const char *moduleName,
     subtree->range_subid = range_subid;
     subtree->range_ubound = range_ubound;
     subtree->session = ss;
-    subtree->reginfo = reginfo;
     subtree->flags = (u_char)flags;    /*  used to identify instance oids  */
     subtree->flags |= SUBTREE_ATTACHED;
     subtree->global_cacheid = reginfo->global_cacheid;
@@ -615,6 +625,8 @@ netsnmp_register_mib(const char *moduleName,
             sub2->name_a[range_subid - 1]  = i;
             sub2->start_a[range_subid - 1] = i;
             sub2->end_a[range_subid - 1]   = i;     /* XXX - ???? */
+            if (range_subid == mibloclen)
+                ++sub2->end_a[range_subid - 1];
             res = netsnmp_subtree_load(sub2, context);
             sub2->flags |= SUBTREE_ATTACHED;
             if (res != MIB_REGISTERED_OK) {
@@ -628,7 +640,10 @@ netsnmp_register_mib(const char *moduleName,
         }
     } else if (res == MIB_DUPLICATE_REGISTRATION ||
                res == MIB_REGISTRATION_FAILED) {
+        netsnmp_set_lookup_cache_size(old_lookup_cache_val);
+        invalidate_lookup_cache(context);
         netsnmp_subtree_free(subtree);
+        return res;
     }
 
     /*
@@ -643,6 +658,7 @@ netsnmp_register_mib(const char *moduleName,
     }
 
     if (perform_callback) {
+        memset(&reg_parms, 0x0, sizeof(reg_parms));
         reg_parms.name = mibloc;
         reg_parms.namelen = mibloclen;
         reg_parms.priority = priority;
@@ -677,6 +693,8 @@ register_mib_reattach_node(netsnmp_subtree *s)
         /*
          * only do registrations that are not the top level nodes 
          */
+        memset(&reg_parms, 0x0, sizeof(reg_parms));
+
         /*
          * XXX: do this better 
          */
@@ -887,7 +905,7 @@ unregister_mib_context(oid * name, size_t len, int priority,
                        const char *context)
 {
     netsnmp_subtree *list, *myptr;
-    netsnmp_subtree *prev, *child;       /* loop through children */
+    netsnmp_subtree *prev, *child, *next; /* loop through children */
     struct register_parameters reg_parms;
     int old_lookup_cache_val = netsnmp_get_lookup_cache_size();
     netsnmp_set_lookup_cache_size(0);
@@ -927,7 +945,8 @@ unregister_mib_context(oid * name, size_t len, int priority,
      *  This should also serve to register ranges.
      */
 
-    for (list = myptr->next; list != NULL; list = list->next) {
+    for (list = myptr->next; list != NULL; list = next) {
+        next = list->next; /* list gets freed sometimes; cache next */
         for (child = list, prev = NULL; child != NULL;
              prev = child, child = child->children) {
             if ((netsnmp_oid_equals(child->name_a, child->namelen,
@@ -943,6 +962,7 @@ unregister_mib_context(oid * name, size_t len, int priority,
     }
     netsnmp_subtree_free(myptr);
 
+    memset(&reg_parms, 0x0, sizeof(reg_parms));
     reg_parms.name = name;
     reg_parms.namelen = len;
     reg_parms.priority = priority;
@@ -1016,6 +1036,7 @@ netsnmp_unregister_mib_table_row(oid * name, size_t len, int priority,
     }
 
     name[var_subid - 1] = range_lbound;
+    memset(&reg_parms, 0x0, sizeof(reg_parms));
     reg_parms.name = name;
     reg_parms.namelen = len;
     reg_parms.priority = priority;
@@ -1072,6 +1093,7 @@ unregister_mibs_by_session(netsnmp_session * ss)
                     (!(!ss || ss->flags & SNMP_FLAGS_SUBSESSION) && child->session &&
                      child->session->subsession == ss)) {
 
+                    memset(&rp, 0x0, sizeof(rp));
                     rp.name = child->name_a;
 		    child->name_a = NULL;
                     rp.namelen = child->namelen;
