@@ -32,6 +32,16 @@
  *  @ingroup leaf
  *  @{
  */
+
+/**
+ * Creates an instance helper handler, calls netsnmp_create_handler, which
+ * then could be registered, using netsnmp_register_handler().
+ *
+ * @param void
+ *
+ * @return Returns a pointer to a netsnmp_mib_handler struct which contains
+ *	the handler's name and the access method
+ */
 netsnmp_mib_handler *
 netsnmp_get_instance_handler(void)
 {
@@ -39,6 +49,24 @@ netsnmp_get_instance_handler(void)
                                   netsnmp_instance_helper_handler);
 }
 
+/**
+ * This function registers an instance helper handler, which is a way of 
+ * registering an exact OID such that GENEXT requests are handled entirely
+ * by the helper. First need to inject it into the calling chain of the 
+ * handler defined by the netsnmp_handler_registration struct, reginfo.  
+ * The new handler is injected at the top of the list and will be the new
+ * handler to be called first.  This function also injects a serialize 
+ * handler before actually calling netsnmp_register_handle, registering 
+ * reginfo.
+ *
+ * @param reginfo a handler registration structure which could get created
+ *                using netsnmp_create_handler_registration.  Used to register
+ *                an instance helper handler.
+ *
+ * @return
+ *      MIB_REGISTERED_OK is returned if the registration was a success.
+ *	Failures are MIB_REGISTRATION_FAILED and MIB_DUPLICATE_REGISTRATION.
+ */
 int
 netsnmp_register_instance(netsnmp_handler_registration *reginfo)
 {
@@ -46,6 +74,24 @@ netsnmp_register_instance(netsnmp_handler_registration *reginfo)
     return netsnmp_register_serialize(reginfo);
 }
 
+/**
+ * This function injects a "read only" handler into the handler chain 
+ * prior to serializing/registering the handler.
+ *
+ * The only purpose of this "read only" handler is to return an
+ * appropriate error for any requests passed to it in a SET mode.
+ * Inserting it into your handler chain will ensure you're never
+ * asked to perform a SET request so you can ignore those error
+ * conditions.
+ *
+ * @param reginfo a handler registration structure which could get created
+ *                using netsnmp_create_handler_registration.  Used to register
+ *                a read only instance helper handler.
+ *
+ * @return
+ *      MIB_REGISTERED_OK is returned if the registration was a success.
+ *	Failures are MIB_REGISTRATION_FAILED and MIB_DUPLICATE_REGISTRATION.
+ */
 int
 netsnmp_register_read_only_instance(netsnmp_handler_registration *reginfo)
 {
@@ -126,7 +172,7 @@ netsnmp_register_read_only_counter32_instance(const char *name,
     netsnmp_handler_registration *myreg;
 
     myreg = get_reg(name, "counter32_handler", reg_oid, reg_oid_len, it,
-                    HANDLER_CAN_RWRITE, netsnmp_instance_counter32_handler,
+                    HANDLER_CAN_RONLY, netsnmp_instance_counter32_handler,
                     subhandler);
     return netsnmp_register_read_only_instance(myreg);
 }
@@ -184,6 +230,24 @@ register_read_only_int_instance(const char *name,
                                 it, subhandler);
 }
 
+/**
+ * This function registers an int helper handler to a specified OID.
+ *
+ * @param name         the name used for registration pruposes.
+ *
+ * @param reg_oid      the OID where you want to register your integer at
+ *
+ * @param reg_oid_len  the length of the OID
+ *
+ * @param it           the integer value to be registered during initialization
+ *
+ * @param subhandler   a handler to do whatever you want to do, otherwise use
+ *                     NULL to use the default int handler.
+ *
+ * @return
+ *      MIB_REGISTERED_OK is returned if the registration was a success.
+ *	Failures are MIB_REGISTRATION_FAILED and MIB_DUPLICATE_REGISTRATION.
+ */
 int
 netsnmp_register_int_instance(const char *name,
                               oid * reg_oid, size_t reg_oid_len,
