@@ -66,7 +66,7 @@ netsnmp_init_table_dataset(void) {
                                 "tableoid");
 #endif /* DISABLE_MIB_LOADING */
     register_app_config_handler("add_row", netsnmp_config_parse_add_row,
-                                NULL, "indexes... values...");
+                                NULL, "table_name indexes... values...");
 }
 
 /** Create a netsnmp_table_data_set structure given a table_data definition */
@@ -346,9 +346,13 @@ netsnmp_table_row *
 netsnmp_table_data_set_clone_row(netsnmp_table_row *row)
 {
     netsnmp_table_data_set_storage *data, **newrowdata;
-    netsnmp_table_row *newrow = netsnmp_table_data_clone_row(row);
+    netsnmp_table_row *newrow;
 
-    if (!row || !newrow)
+    if (!row)
+        return NULL;
+
+    newrow = netsnmp_table_data_clone_row(row);
+    if (!newrow)
         return NULL;
 
     data = (netsnmp_table_data_set_storage *) row->data;
@@ -360,14 +364,18 @@ netsnmp_table_data_set_clone_row(netsnmp_table_row *row)
 
             memdup((u_char **) newrowdata, (u_char *) data,
                    sizeof(netsnmp_table_data_set_storage));
-            if (!*newrowdata)
+            if (!*newrowdata) {
+                netsnmp_table_dataset_delete_row(newrow);
                 return NULL;
+            }
 
             if (data->data.voidp) {
                 memdup((u_char **) & ((*newrowdata)->data.voidp),
                        (u_char *) data->data.voidp, data->data_len);
-                if (!(*newrowdata)->data.voidp)
+                if (!(*newrowdata)->data.voidp) {
+                    netsnmp_table_dataset_delete_row(newrow);
                     return NULL;
+                }
             }
         }
     }
@@ -451,6 +459,10 @@ netsnmp_table_data_set_helper_handler(netsnmp_mib_handler *handler,
             (reginfo->rootoid_len + 2);
 
         if (MODE_IS_SET(reqinfo->mode)) {
+            char buf[256]; /* is this reasonable size?? */
+            int  rc;
+            size_t len;
+
             /*
              * use a cached copy of the row for modification 
              */
@@ -459,9 +471,27 @@ netsnmp_table_data_set_helper_handler(netsnmp_mib_handler *handler,
              * cache location: may have been created already by other
              * SET requests in the same master request. 
              */
+            rc = snprintf(buf, sizeof(buf), "dataset_row_stash:%s:",
+                          datatable->table->name);
+            if ((-1 == rc) || (rc >= sizeof(buf))) {
+                snmp_log(LOG_ERR,"%s handler name too long\n",
+                         datatable->table->name);
+                netsnmp_set_request_error(reqinfo, request,
+                                          SNMP_ERR_GENERR);
+                continue;
+            }
+            len = sizeof(buf) - rc;
+            rc = snprint_objid(&buf[rc], len, table_info->index_oid,
+                               table_info->index_oid_len);
+            if (-1 == rc) {
+                snmp_log(LOG_ERR,"%s oid or name too long\n",
+                         datatable->table->name);
+                netsnmp_set_request_error(reqinfo, request,
+                                          SNMP_ERR_GENERR);
+                continue;
+            }
             stashp = (netsnmp_oid_stash_node **)
-                netsnmp_table_get_or_create_row_stash(reqinfo,
-                                                      "dataset_row_stash");
+                netsnmp_table_get_or_create_row_stash(reqinfo, buf);
 
             newrowstash
                 = netsnmp_oid_stash_get_data(*stashp, suffix, suffix_len);
@@ -773,29 +803,102 @@ netsnmp_config_parse_table_set(const char *token, char *line)
         MAX_OID_LEN;
     struct tree    *tp, *indexnode;
     netsnmp_table_data_set *table_set;
+    data_set_tables *tables;
     struct index_list *index;
     unsigned int    mincol = 0xffffff, maxcol = 0;
     u_char          type;
+    char           *pos;
 
     /*
      * instatiate a fake table based on MIB information 
      */
-    if (!snmp_parse_oid(line, table_name, &table_name_length) ||
-        (NULL == (tp = get_tree(table_name, table_name_length,
-                                get_tree_head())))) {
+    DEBUGMSGTL(("9:table_set_add_table", "processing '%s'\n", line));
+    if (NULL != (pos = strchr(line,' '))) {
+        config_pwarn("ignoring extra tokens on line");
+        snmp_log(LOG_WARNING,"  ignoring '%s'\n", pos);
+        *pos = '\0';
+    }
+
+    /*
+     * check for duplicate table
+     */
+    tables = (data_set_tables *) netsnmp_get_list_data(auto_tables, line);
+    if (NULL != tables) {
+        config_pwarn("duplicate table definition");
+        return;
+    }
+
+    /*
+     * parse oid and find tree structure
+     */
+    if (!snmp_parse_oid(line, table_name, &table_name_length)) {
         config_pwarn
-            ("can't instatiate table %s since I can't find mib information about it\n");
+            ("can't instatiate table since I can't parse the table name");
+        return;
+    }
+    if(NULL == (tp = get_tree(table_name, table_name_length,
+                              get_tree_head()))) {
+        config_pwarn("can't instatiate table since "
+                     "I can't find mib information about it");
         return;
     }
 
     if (NULL == (tp = tp->child_list) || NULL == tp->child_list) {
-        config_pwarn
-            ("can't instatiate table since it doesn't appear to be a proper table\n");
+        config_pwarn("can't instatiate table since it doesn't appear to be "
+                     "a proper table (no children)");
         return;
     }
 
-    table_set = netsnmp_create_table_data_set(line);
+    /*
+     * check for augments indexes
+     */
+    if (NULL != tp->augments) {
+        if (!snmp_parse_oid(tp->augments, table_name, &table_name_length)) {
+            config_pwarn("I can't parse the augment tabel name");
+            snmp_log(LOG_WARNING, "  can't parse %s\n", tp->augments);
+            return;
+        }
+        if(NULL == (tp = get_tree(table_name, table_name_length,
+                                  get_tree_head()))) {
+            config_pwarn("can't instatiate table since "
+                         "I can't find mib information about augment table");
+            snmp_log(LOG_WARNING, "  table %s not found in tree\n",
+                     tp->augments);
+            return;
+        }
 
+        table_set = netsnmp_create_table_data_set(line);
+    
+        /*
+         * loop through indexes and add types 
+         */
+        for (index = tp->indexes; index; index = index->next) {
+            if (!snmp_parse_oid(index->ilabel, name, &name_length) ||
+                (NULL ==
+                 (indexnode = get_tree(name, name_length, get_tree_head())))) {
+                config_pwarn("can't instatiate table since "
+                             "I don't know anything about one index");
+                snmp_log(LOG_WARNING, "  index %s not found in tree\n",
+                         index->ilabel);
+                return;             /* xxx mem leak */
+            }
+            
+            type = mib_to_asn_type(indexnode->type);
+            if (type == (u_char) - 1) {
+                config_pwarn("unknown index type");
+                return;             /* xxx mem leak */
+            }
+            if (index->isimplied)   /* if implied, mark it as such */
+                type |= ASN_PRIVATE;
+            
+            DEBUGMSGTL(("table_set_add_row",
+                        "adding default index of type %d\n", type));
+            netsnmp_table_dataset_add_index(table_set, type);
+        }
+    }
+    else
+        table_set = netsnmp_create_table_data_set(line);
+    
     /*
      * loop through indexes and add types 
      */
@@ -803,8 +906,10 @@ netsnmp_config_parse_table_set(const char *token, char *line)
         if (!snmp_parse_oid(index->ilabel, name, &name_length) ||
             (NULL ==
              (indexnode = get_tree(name, name_length, get_tree_head())))) {
-            config_pwarn
-                ("can't instatiate table %s since I don't know anything about one index\n");
+            config_pwarn("can't instatiate table since "
+                         "I don't know anything about one index");
+            snmp_log(LOG_WARNING, "  index %s not found in tree\n",
+                     index->ilabel);
             return;             /* xxx mem leak */
         }
 
@@ -832,8 +937,9 @@ netsnmp_config_parse_table_set(const char *token, char *line)
             return;             /* xxx mem leak */
         }
 
-        DEBUGMSGTL(("table_set_add_row", "adding column %d of type %d\n",
-                    tp->subid, type));
+        DEBUGMSGTL(("table_set_add_row",
+                    "adding column %s(%d) of type %d (access %d)\n",
+                    tp->label, tp->subid, type, tp->access));
 
         switch (tp->access) {
         case MIB_ACCESS_CREATE:
@@ -879,6 +985,7 @@ netsnmp_config_parse_add_row(const char *token, char *line)
     char            buf[SNMP_MAXBUF_MEDIUM];
     char            tname[SNMP_MAXBUF_MEDIUM];
     size_t          buf_size;
+    int             rc;
 
     data_set_tables *tables;
     netsnmp_variable_list *vb;  /* containing only types */
@@ -917,7 +1024,10 @@ netsnmp_config_parse_add_row(const char *token, char *line)
      */
     for (dr = tables->table_set->default_row; dr; dr = dr->next) {
         if (!line) {
-            config_pwarn("missing an data value\n");
+            config_pwarn("missing a data value. "
+                         "All columns must be specified.");
+            snmp_log(LOG_WARNING,"  can't find value for column %d\n",
+                     dr->column - 1);
             return;
         }
 
@@ -930,7 +1040,10 @@ netsnmp_config_parse_add_row(const char *token, char *line)
         if (dr->writable)
             netsnmp_mark_row_column_writable(row, dr->column, 1);       /* make writable */
     }
-    netsnmp_table_data_add_row(tables->table_set->table, row);
+    rc = netsnmp_table_data_add_row(tables->table_set->table, row);
+    if (SNMPERR_SUCCESS != rc) {
+        config_pwarn("error adding table row");
+    }
 }
 
 /** adds an index to the table.  Call this repeatly for each index. */
