@@ -94,7 +94,6 @@ static int ps_numdisks;			/* number of disks in system, may change while running
 static mach_port_t masterPort;		/* to communicate with I/O Kit	*/
 #endif                          /* darwin */
 
-static char     type[20];
 void            diskio_parse_config(const char *, char *);
 FILE           *file;
 
@@ -161,12 +160,6 @@ init_diskio(void)
     REGISTER_MIB("diskio", diskio_variables, variable2,
                  diskio_variables_oid);
 
-    /*
-     * Added to parse snmpd.conf - abby
-     */
-    snmpd_register_config_handler("diskio", diskio_parse_config,
-                                  NULL, "diskio [device-type]");
-
 #ifdef solaris2
     kc = kstat_open();
 
@@ -190,12 +183,6 @@ init_diskio(void)
 #endif
 }
 
-void
-diskio_parse_config(const char *token, char *cptr)
-{
-    copy_nword(cptr, type, sizeof(type));
-}
-
 #ifdef solaris2
 int
 get_disk(int disknr)
@@ -422,8 +409,8 @@ getstats(void)
         	fprintf(stderr, "Memory alloc error - devinfo\n");
 		return 1;
 	}
+        memset(stat->dinfo, 0, sizeof(struct devinfo));
     }
-    memset(stat->dinfo, 0, sizeof(struct devinfo));
 
     if (GETDEVS(stat) == -1) {
         fprintf(stderr, "Can't get devices:%s\n", devstat_errbuf);
