@@ -228,7 +228,7 @@ get_exec_output(struct extensible *ex)
 #ifdef EXCACHETIME
     char            cachefile[STRMAX];
     char            cache[MAXCACHESIZE];
-    size_t          cachebytes;
+    ssize_t         cachebytes;
     long            curtime;
     static char     lastcmd[STRMAX];
     int             cfd;
@@ -824,7 +824,7 @@ struct internal_mib_table {
 };
 
 mib_table_t
-Initialise_Table(int size, int timeout, RELOAD reload, COMPARE compare)
+Initialise_Table(int size, int timeout, RELOAD *reload, COMPARE *compare)
 {
     struct internal_mib_table *t;
 
