@@ -189,7 +189,9 @@ int             Facility = LOG_DAEMON;
 #define AGENT_RUNNING 1
 #define AGENT_STOPPED 0
 int             agent_status = AGENT_STOPPED;
-LPTSTR          g_szAppName = _T("Net-SNMP Agent");     /* Application Name */
+LPTSTR          app_name = _T("Net-SNMP Agent");     /* Application Name */
+#else
+const char     *app_name = "snmpd";
 #endif
 
 extern char   **argvrestartp;
@@ -261,9 +263,9 @@ static void
 usage(char *prog)
 {
 #ifdef WIN32SERVICE
-    printf("\nUsage:  %s [-register] [OPTIONS] [LISTENING ADDRESSES]",
+    printf("\nUsage:  %s [-register] [-quiet] [OPTIONS] [LISTENING ADDRESSES]",
            prog);
-    printf("\n        %s -unregister", prog);
+    printf("\n        %s [-unregister] [-quiet]", prog);
 #else
     printf("\nUsage:  %s [OPTIONS] [LISTENING ADDRESSES]", prog);
 #endif
@@ -296,8 +298,9 @@ usage(char *prog)
 	   "\t\t\t  cannot be opened\n");
 #ifdef WIN32SERVICE
     printf("  -register\t\tregister as a Windows service\n");
+    printf("  \t\t\t  (followed by -quiet to prevent message popups)\n");
     printf("  \t\t\t  (followed by the startup parameter list)\n");
-    printf("  \t\t\t  Note that not all parameters are relevant when running as a service\n");
+    printf("  \t\t\t  Note that some parameters are not relevant when running as a service\n");
 #endif
 #if HAVE_UNISTD_H
     printf("  -u UID\t\tchange to this uid (numeric or textual) after\n"
@@ -305,6 +308,7 @@ usage(char *prog)
 #endif
 #ifdef WIN32SERVICE
     printf("  -unregister\t\tunregister as a Windows service\n");
+    printf("  \t\t\t  (followed -quiet to prevent message popups)\n");
 #endif
     printf("  -v, --version\t\tdisplay version information\n");
     printf("  -V\t\t\tverbose display\n");
@@ -391,13 +395,17 @@ SnmpTrapNodeDown(void)
      */
 }
 
+/*
+ * dummy1 used to be stderr_log, but that was never set. So, ignore the
+ * value here, instead of always disabling stderr logging (which might
+ * have been set up using the new log handler methods).
+ */
 static void
-setup_log(int restart, int dont_zero, int stderr_log, int syslog_log, 
+setup_log(int restart, int dont_zero, int dummy1, int syslog_log, 
 	  char *logfile)
 {
     static char logfile_s[PATH_MAX + 1] = { 0 };
     static int dont_zero_s  = 0;
-    static int stderr_log_s = 0;
     static int syslog_log_s = 0;
 
     if (restart == 0) {
@@ -405,28 +413,21 @@ setup_log(int restart, int dont_zero, int stderr_log, int syslog_log,
 	    strncpy(logfile_s, logfile, PATH_MAX);
 	}
 	dont_zero_s  = dont_zero;
-	stderr_log_s = stderr_log;
 	syslog_log_s = syslog_log;
     }
 
-    if (stderr_log_s) {
-	snmp_enable_stderrlog();
-    } else {
-	snmp_disable_stderrlog();
-    }
-
     if (logfile_s[0]) {
 	snmp_enable_filelog(logfile_s, dont_zero_s);
     }
 
     if (syslog_log_s) {
-	snmp_enable_syslog_ident("snmpd", Facility);
+	snmp_enable_syslog_ident(app_name, Facility);
     }
 }
 
 /*******************************************************************-o-******
  * main - Non Windows
- * SnmpDeamonMain - Windows to support windows serivce
+ * SnmpDaemonMain - Windows to support windows service
  *
  * Parameters:
  *	 argc
@@ -451,7 +452,7 @@ main(int argc, char *argv[])
     int             arg, i, ret;
     int             dont_fork = 0;
     int             dont_zero_log = 0;
-    int             stderr_log = 0, syslog_log = 0;
+    int             syslog_log = 0;
     int             uid = 0, gid = 0;
     int             agent_mode = -1;
     char            logfile[PATH_MAX + 1] = { 0 };
@@ -463,6 +464,30 @@ main(int argc, char *argv[])
     FILE           *PID;
 #endif
 
+#ifdef SIGTERM
+    DEBUGMSGTL(("signal", "registering SIGTERM signal handler\n"));
+    signal(SIGTERM, SnmpdShutDown);
+#endif
+#ifdef SIGINT
+    DEBUGMSGTL(("signal", "registering SIGINT signal handler\n"));
+    signal(SIGINT, SnmpdShutDown);
+#endif
+#ifdef SIGHUP
+    DEBUGMSGTL(("signal", "registering SIGHUP signal handler\n"));
+    signal(SIGHUP, SnmpdReconfig);
+#endif
+#ifdef SIGUSR1
+    DEBUGMSGTL(("signal", "registering SIGUSR1 signal handler\n"));
+    signal(SIGUSR1, SnmpdDump);
+#endif
+#ifdef SIGPIPE
+    DEBUGMSGTL(("signal", "registering SIGPIPE signal handler\n"));
+    signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
+#endif
+#ifdef SIGXFSZ
+    signal(SIGXFSZ, SnmpdCatchRandomSignal);
+#endif
+
 #ifdef LOGFILE
     strncpy(logfile, LOGFILE, PATH_MAX);
 #endif
@@ -506,6 +531,8 @@ main(int argc, char *argv[])
             argv[i] = option_compatability;            
     }
 
+    snmp_log_syslogname(app_name);
+
     /*
      * Now process options normally.  
      */
@@ -608,9 +635,6 @@ main(int argc, char *argv[])
             break;
 
         case 'L':
-	    /*
-            stderr_log = 1;
-	     */
 	    if  (snmp_log_options( optarg, argc, argv ) < 0 ) {
                 usage(argv[0]);
             }
@@ -783,7 +807,7 @@ main(int argc, char *argv[])
             char *c, *astring;
             if ((c = netsnmp_ds_get_string(NETSNMP_DS_APPLICATION_ID, 
 					   NETSNMP_DS_AGENT_PORTS))) {
-                astring = malloc(strlen(c) + 2 + strlen(argv[i]));
+                astring = (char*)malloc(strlen(c) + 2 + strlen(argv[i]));
                 if (astring == NULL) {
                     fprintf(stderr, "malloc failure processing argv[%d]\n", i);
                     exit(1);
@@ -802,7 +826,7 @@ main(int argc, char *argv[])
 					  NETSNMP_DS_AGENT_PORTS)));
     }
 
-    setup_log(0, dont_zero_log, stderr_log, syslog_log, logfile);
+    setup_log(0, dont_zero_log, 0, syslog_log, logfile);
 
     /*
      * Initialize a argv set to the current for restarting the agent.   
@@ -849,7 +873,7 @@ main(int argc, char *argv[])
     if(!dont_fork) {
         int quit = ! netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
                                             NETSNMP_DS_AGENT_QUIT_IMMEDIATELY);
-        ret = netsnmp_daemonize(quit, stderr_log);
+        ret = netsnmp_daemonize(quit, snmp_stderrlog_status());
         /*
          * xxx-rks: do we care if fork fails? I think we should...
          */
@@ -872,29 +896,6 @@ main(int argc, char *argv[])
          */
         Exit(1);                /*  Exit logs exit val for us  */
     }
-#ifdef SIGTERM
-    DEBUGMSGTL(("signal", "registering SIGTERM signal handler\n"));
-    signal(SIGTERM, SnmpdShutDown);
-#endif
-#ifdef SIGINT
-    DEBUGMSGTL(("signal", "registering SIGINT signal handler\n"));
-    signal(SIGINT, SnmpdShutDown);
-#endif
-#ifdef SIGHUP
-    DEBUGMSGTL(("signal", "registering SIGHUP signal handler\n"));
-    signal(SIGHUP, SnmpdReconfig);
-#endif
-#ifdef SIGUSR1
-    DEBUGMSGTL(("signal", "registering SIGUSR1 signal handler\n"));
-    signal(SIGUSR1, SnmpdDump);
-#endif
-#ifdef SIGPIPE
-    DEBUGMSGTL(("signal", "registering SIGPIPE signal handler\n"));
-    signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
-#endif
-#ifdef SIGXFSZ
-    signal(SIGXFSZ, SnmpdCatchRandomSignal);
-#endif
 
     /*
      * Store persistent data immediately in case we crash later.  
@@ -1047,6 +1048,11 @@ receive(void)
         svp->tv_sec++;
     }
 
+    /*
+     * ignore early sighup during startup
+     */
+    reconfig = 0;
+
     /*
      * Loop-forever: execute message handlers for sockets with data,
      * reset the 'sched'uler.
@@ -1227,9 +1233,6 @@ receive(void)
                 svp->tv_usec -= ONE_SEC;
                 svp->tv_sec++;
             }
-            if (log_addresses && lastAddrAge++ > 600) {
-                netsnmp_addrcache_age();
-            }
         }
 
         /*
@@ -1306,8 +1309,8 @@ snmp_input(int op,
 * main function for Windows
 * Parse command line arguments for startup options,
 * to start as service or console mode application in windows.
-* Invokes appropriate startup funcitons depending on the 
-* parameters passesd
+* Invokes appropriate startup functions depending on the 
+* parameters passed
 *************************************************************/
 int
     __cdecl
@@ -1316,7 +1319,7 @@ _tmain(int argc, TCHAR * argv[])
     /*
      * Define Service Name and Description, which appears in windows SCM 
      */
-    LPCTSTR         lpszServiceName = g_szAppName;      /* Service Registry Name */
+    LPCTSTR         lpszServiceName = app_name;      /* Service Registry Name */
     LPCTSTR         lpszServiceDisplayName = _T("Net-SNMP Agent");       /* Display Name */
     LPCTSTR         lpszServiceDescription =
 #ifdef IFDESCR
@@ -1328,7 +1331,9 @@ _tmain(int argc, TCHAR * argv[])
 
 
     int             nRunType = RUN_AS_CONSOLE;
-    nRunType = ParseCmdLineForServiceOption(argc, argv);
+    int             quiet = 0;
+    
+    nRunType = ParseCmdLineForServiceOption(argc, argv, &quiet);
 
     switch (nRunType) {
     case REGISTER_SERVICE:
@@ -1337,17 +1342,15 @@ _tmain(int argc, TCHAR * argv[])
          */
         InputOptions.Argc = argc;
         InputOptions.Argv = argv;
-        RegisterService(lpszServiceName,
+        exit (RegisterService(lpszServiceName,
                         lpszServiceDisplayName,
-                        lpszServiceDescription, &InputOptions);
-        exit(0);
+                        lpszServiceDescription, &InputOptions, quiet));
         break;
     case UN_REGISTER_SERVICE:
         /*
          * Unregister service 
          */
-        UnregisterService(lpszServiceName);
-        exit(0);
+        exit (UnregisterService(lpszServiceName, quiet));
         break;
     case RUN_AS_SERVICE:
         /*
@@ -1361,10 +1364,7 @@ _tmain(int argc, TCHAR * argv[])
         break;
     default:
         /*
-         * Run Net-Snmpd in console mode 
-         */
-        /*
-         * Invoke SnmpDeamonMain with input arguments 
+         * Run in console mode 
          */
         return SnmpDaemonMain(argc, argv);
         break;
@@ -1372,7 +1372,7 @@ _tmain(int argc, TCHAR * argv[])
 }
 
 /*
- * To stop Snmp Agent deamon 
+ * To stop Snmp Agent daemon
  * This portion is still not working
  */
 void
@@ -1392,4 +1392,4 @@ StopSnmpAgent(void)
     }
 }
 
-#endif/*WIN32SERVICE*/
+#endif /*WIN32SERVICE*/
