@@ -10,6 +10,9 @@
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
+
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+
 #include <net-snmp/agent/instance.h>
 #include <net-snmp/agent/table.h>
 #include <net-snmp/agent/table_data.h>
@@ -703,3 +706,4 @@ int   notification_handler(netsnmp_pdu           *pdu,
     log_notification(host, pdu, transport);
     return NETSNMPTRAPD_HANDLER_OK;
 }
+#endif /* USING_AGENTX_SUBAGENT_MODULE && !SNMPTRAPD_DISABLE_AGENTX */
