@@ -199,7 +199,9 @@ int Facility = LOG_DAEMON;
 #define SNMPTRAPD_RUNNING 1
 #define SNMPTRAPD_STOPPED 0
 int             trapd_status = SNMPTRAPD_STOPPED;
-LPTSTR          g_szAppName = _T("Net-SNMP Trap Handler");     /* Application Name */
+LPTSTR          app_name = _T("Net-SNMP Trap Handler");     /* Application Name */
+#else
+const char     *app_name = "snmptrapd";
 #endif
 
 struct timeval  Now;
@@ -289,8 +291,8 @@ void
 usage(void)
 {
 #ifdef WIN32SERVICE
-    fprintf(stderr, "\nUsage:  snmptrapd [-register] [OPTIONS] [LISTENING ADDRESSES]");
-    fprintf(stderr, "\n        snmptrapd -unregister");
+    fprintf(stderr, "\nUsage:  snmptrapd [-register] [-quiet] [OPTIONS] [LISTENING ADDRESSES]");
+    fprintf(stderr, "\n        snmptrapd [-unregister] [-quiet]");
 #else
     fprintf(stderr, "Usage: snmptrapd [OPTIONS] [LISTENING ADDRESSES]\n");
 #endif
@@ -323,12 +325,14 @@ usage(void)
 #endif
 #ifdef WIN32SERVICE
     fprintf(stderr, "  -register\t\tregister as a Windows service\n");
+    fprintf(stderr, "  \t\t\t  (followed by -quiet to prevent message popups)\n");
     fprintf(stderr, "  \t\t\t  (followed by the startup parameter list)\n");
-    fprintf(stderr, "  \t\t\t  Note that not all parameters are relevant when running as a service\n");
+    fprintf(stderr, "  \t\t\t  Note that some parameters are not relevant when running as a service\n");
 #endif
     fprintf(stderr, "  -t\t\t\tPrevent traps from being logged to syslog\n");
 #ifdef WIN32SERVICE
     fprintf(stderr, "  -unregister\t\tunregister as a Windows service\n");
+    fprintf(stderr, "  \t\t\t  (followed -quiet to prevent message popups)\n");
 #endif
     fprintf(stderr, "  -v, --version\t\tdisplay version information\n");
     fprintf(stderr,
@@ -399,6 +403,13 @@ pre_parse(netsnmp_session * session, netsnmp_transport *transport,
     }
 
     if (addr_string != NULL) {
+        if( addr_string[0] == '[' ) { /* fix up ipv6 addr */
+            int i;
+            for( i = 1; addr_string[i] != ']'; i++ ) {
+                addr_string[i-1] =  addr_string[i];
+            }
+            addr_string[i-1] = '\0';
+        }
         if (hosts_ctl("snmptrapd", STRING_UNKNOWN, 
 		      addr_string, STRING_UNKNOWN) == 0) {
             free(addr_string);
@@ -475,7 +486,7 @@ free_trapd_address(void)
 
 /*******************************************************************-o-******
  * main - Non Windows
- * SnmpTrapdMain - Windows to support windows serivce
+ * SnmpTrapdMain - Windows to support windows service
  *
  * Parameters:
  *	 argc
@@ -496,22 +507,37 @@ SnmpTrapdMain(int argc, TCHAR * argv[])
 main(int argc, char *argv[])
 #endif
 {
-    char            options[128] = "ac:CdD::efF:hHl:L:m:M:no:PqsS:tvO:-:";
+    char            options[128] = "ac:CdD::efF:hHI:l:L:m:M:no:PqsS:tvO:-:";
     netsnmp_session *sess_list = NULL, *ss = NULL;
     netsnmp_transport *transport = NULL;
-    int             arg, i = 0;
+    int             arg, i = 0, depmsg = 0;
     int             count, numfds, block;
     fd_set          fdset;
     struct timeval  timeout, *tvp;
     int             dofork = 1;
     char           *cp, *listen_ports = NULL;
     char           *trap1_fmt_str_remember = NULL;
-    int             agentx_subagent = 1, depmsg = 0;
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+    int             agentx_subagent = 1;
+#endif
 #if HAVE_GETPID
     FILE           *PID;
     char           *pid_file = NULL;
 #endif
 
+#ifdef SIGTERM
+    signal(SIGTERM, term_handler);
+#endif
+#ifdef SIGHUP
+    signal(SIGHUP, hup_handler);
+#endif
+#ifdef SIGINT
+    signal(SIGINT, term_handler);
+#endif
+#ifdef SIGPIPE
+    signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
+#endif
+
     /*
      * register our configuration handlers now so -H properly displays them 
      */
@@ -543,6 +569,8 @@ main(int argc, char *argv[])
     strcat(options, "p:u:");
 #endif
 
+    snmp_log_syslogname(app_name);
+
     /*
      * Now process options normally.  
      */
@@ -612,11 +640,22 @@ main(int argc, char *argv[])
             exit(0);
 
         case 'H':
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
             init_notification_log();
+#endif
             init_snmp("snmptrapd");
             fprintf(stderr, "Configuration directives understood:\n");
             read_config_print_usage("  ");
             exit(0);
+            break;
+
+        case 'I':
+            if (optarg != NULL) {
+                add_to_init_list(optarg);
+            } else {
+                usage();
+            }
+            break;
 
 	case 'S':
             fprintf(stderr,
@@ -805,7 +844,6 @@ main(int argc, char *argv[])
     } else {
         netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER, print_handler);
     }
-    netsnmp_add_global_traphandler(NETSNMPTRAPD_POST_HANDLER, notification_handler);
 
     if (Event) {
         netsnmp_add_traphandler(event_handler, risingAlarm,
@@ -820,7 +858,7 @@ main(int argc, char *argv[])
 	 */
     }
 
-#ifdef USING_AGENTX_SUBAGENT_MODULE
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
     /*
      * we're an agentx subagent? 
      */
@@ -849,16 +887,20 @@ main(int argc, char *argv[])
      */
     init_agent("snmptrapd");
 
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
     /*
      * initialize local modules 
      */
     if (agentx_subagent) {
-#ifdef USING_AGENTX_SUBAGENT_MODULE
 	void  init_subagent(void);
         init_subagent();
-#endif
-        init_notification_log();
+        if (should_init("notificationLogMib")) {
+            netsnmp_add_global_traphandler(NETSNMPTRAPD_POST_HANDLER,
+                                           notification_handler);
+            init_notification_log();
+        }
     }
+#endif
 
     /*
      * Initialize the world. Create initial user 
@@ -927,7 +969,7 @@ main(int argc, char *argv[])
 #endif
 
     if (Syslog) {
-        snmp_enable_syslog_ident("snmptrapd", Facility);
+        snmp_enable_syslog_ident(app_name, Facility);
         snmp_log(LOG_INFO, "Starting snmptrapd %s\n", netsnmp_get_version());
 	if (depmsg) {
 	    snmp_log(LOG_WARNING, "-s and -S options are deprecated; use -Ls <facility> instead\n");
@@ -1009,18 +1051,17 @@ main(int argc, char *argv[])
         }
     }
 
-    signal(SIGTERM, term_handler);
-#ifdef SIGHUP
-    signal(SIGHUP, hup_handler);
-#endif
-    signal(SIGINT, term_handler);
+    /*
+     * ignore early sighup during startup
+     */
+    reconfig = 0;
 
 #ifdef WIN32SERVICE
     trapd_status = SNMPTRAPD_RUNNING;
 #endif
     while (running) {
         if (reconfig) {
-            if (Print) {
+            if (Print || Log) {
                 struct tm      *tm;
                 time_t          timer;
                 time(&timer);
@@ -1081,7 +1122,7 @@ main(int argc, char *argv[])
 	run_alarms();
     }
 
-    if (Print) {
+    if (Print || Log) {
         struct tm      *tm;
         time_t          timer;
         time(&timer);
@@ -1108,7 +1149,7 @@ main(int argc, char *argv[])
 /*
  * Read the configuration files. Implemented as a signal handler so that
  * receipt of SIGHUP will cause configuration to be re-read when the
- * trap deamon is running detatched from the console.
+ * trap daemon is running detatched from the console.
  *
  */
 void
@@ -1138,8 +1179,8 @@ getdtablesize(void)
 * main function for Windows
 * Parse command line arguments for startup options,
 * to start as service or console mode application in windows.
-* Invokes appropriate startup funcitons depending on the 
-* parameters passesd
+* Invokes appropriate startup functions depending on the 
+* parameters passed
 *************************************************************/
 int
     __cdecl
@@ -1148,7 +1189,7 @@ _tmain(int argc, TCHAR * argv[])
     /*
      * Define Service Name and Description, which appears in windows SCM 
      */
-    LPCTSTR         lpszServiceName = g_szAppName;      /* Service Registry Name */
+    LPCTSTR         lpszServiceName = app_name;      /* Service Registry Name */
     LPCTSTR         lpszServiceDisplayName = _T("Net-SNMP Trap Handler");       /* Display Name */
     LPCTSTR         lpszServiceDescription =
 #ifdef IFDESCR
@@ -1158,9 +1199,10 @@ _tmain(int argc, TCHAR * argv[])
 #endif
     InputParams     InputOptions;
 
-
     int             nRunType = RUN_AS_CONSOLE;
-    nRunType = ParseCmdLineForServiceOption(argc, argv);
+    int             quiet = 0;
+
+    nRunType = ParseCmdLineForServiceOption(argc, argv, &quiet);
 
     switch (nRunType) {
     case REGISTER_SERVICE:
@@ -1169,16 +1211,15 @@ _tmain(int argc, TCHAR * argv[])
          */
         InputOptions.Argc = argc;
         InputOptions.Argv = argv;
-        RegisterService(lpszServiceName,
+        exit (RegisterService(lpszServiceName,
                         lpszServiceDisplayName,
-                        lpszServiceDescription, &InputOptions);
-        exit(0);
+                        lpszServiceDescription, &InputOptions, quiet));
         break;
     case UN_REGISTER_SERVICE:
         /*
          * Unregister service 
          */
-        UnregisterService(lpszServiceName);
+        exit (UnregisterService(lpszServiceName, quiet));
         exit(0);
         break;
     case RUN_AS_SERVICE:
@@ -1193,10 +1234,7 @@ _tmain(int argc, TCHAR * argv[])
         break;
     default:
         /*
-         * Run Net-Snmpd in console mode 
-         */
-        /*
-         * Invoke SnmpDeamonMain with input arguments 
+         * Run in console mode 
          */
         return SnmpTrapdMain(argc, argv);
         break;
@@ -1204,7 +1242,7 @@ _tmain(int argc, TCHAR * argv[])
 }
 
 /*
- * To stop Snmp Trap Receiver deamon 
+ * To stop Snmp Trap Receiver daemon
  * This portion is still not working
  */
 void
@@ -1224,4 +1262,4 @@ StopSnmpTrapd(void)
     }
 }
 
-#endif/*WIN32SERVICE*/
+#endif /*WIN32SERVICE*/
