@@ -281,25 +281,75 @@ snmp_unregister_callback(int major, int minor, SNMPCallback * target,
     return count;
 }
 
+/*
+ * find and clear client args that match ptr
+ *
+ * @param ptr  pointer to search for
+ * @param i    callback id to start at
+ * @param j    callback subid to start at
+ */
+int
+netsnmp_callback_clear_client_arg(void *ptr, int i, int j)
+{
+    struct snmp_gen_callback *scp = NULL;
+    int rc = 0;
+
+    for (; i < MAX_CALLBACK_IDS; i++) {
+        for (; j < MAX_CALLBACK_SUBIDS; j++) {
+            scp = thecallbacks[i][j]; 
+            while (scp != NULL) {
+                if ((NULL != scp->sc_callback) &&
+                    (scp->sc_client_arg != NULL) &&
+                    (scp->sc_client_arg == ptr)) {
+                    scp->sc_client_arg = NULL;
+                    ++rc;
+                }
+                scp = scp->next;
+            }
+        }
+    }
+
+    if (0 != rc) {
+        DEBUGMSGTL(("callback", "removed %d client args\n", rc));
+    }
+
+    return rc;
+}
+
 void
 clear_callback(void)
 {
     unsigned int i = 0, j = 0; 
-    struct snmp_gen_callback *scp = NULL, *next = NULL;
+    struct snmp_gen_callback *scp = NULL;
 
     DEBUGMSGTL(("callback", "clear callback\n"));
     for (i = 0; i < MAX_CALLBACK_IDS; i++) {
-	for (j = 0; j < MAX_CALLBACK_SUBIDS; j++) {
-	    scp = thecallbacks[i][j]; 
-	    while (scp != NULL) {
-		next = scp->next;
-		if (scp->sc_client_arg != NULL)
-		    SNMP_FREE(scp->sc_client_arg);
-		SNMP_FREE(scp);
-		scp = next;
-	    }
-	    thecallbacks[i][j] = NULL;
-	}
+        for (j = 0; j < MAX_CALLBACK_SUBIDS; j++) {
+            scp = thecallbacks[i][j]; 
+            while (scp != NULL) {
+                thecallbacks[i][j] = scp->next;
+                /*
+                 * if there is a client arg, check for duplicates
+                 * and then free it.
+                 */
+               if ((NULL != scp->sc_callback) &&
+                    (scp->sc_client_arg != NULL)) {
+                    void *tmp_arg;
+                    /*
+                     * save the client arg, then set it to null so that it
+                     * won't look like a duplicate, then check for duplicates
+                     * starting at the current i,j (earlier dups should have
+                     * already been found) and free the pointer.
+                     */
+                   tmp_arg = scp->sc_client_arg;
+                   scp->sc_client_arg = NULL;
+                   netsnmp_callback_clear_client_arg(tmp_arg, i, j);
+                   free(tmp_arg);
+               }
+               SNMP_FREE(scp);
+               scp = thecallbacks[i][j];
+            }
+        }
     }
 }
 
