@@ -201,6 +201,7 @@ register_config_handler(const char *type_param,
     struct config_files **ctmp = &config_files;
     struct config_line **ltmp;
     const char     *type = type_param;
+    char           *st;
 
     if (type == NULL) {
         type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
@@ -285,6 +286,7 @@ unregister_config_handler(const char *type_param, const char *token)
     struct config_files **ctmp = &config_files;
     struct config_line **ltmp, *ltmp2;
     const char     *type = type_param;
+    char           *st;
 
     if (type == NULL) {
         type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
@@ -397,17 +399,23 @@ read_config_get_handlers(const char *type)
 }
 
 void
-read_config_with_type(const char *filename, const char *type)
+read_config_with_type_when(const char *filename, const char *type, int when)
 {
     struct config_line *ctmp = read_config_get_handlers(type);
     if (ctmp)
-        read_config(filename, ctmp, EITHER_CONFIG);
+        read_config(filename, ctmp, when);
     else
         DEBUGMSGTL(("read_config",
                     "read_config: I have no registrations for type:%s,file:%s\n",
                     type, filename));
 }
 
+void
+read_config_with_type(const char *filename, const char *type)
+{
+    read_config_with_type_when(filename, type, EITHER_CONFIG);
+}
+
 
 struct config_line *
 read_config_find_handler(struct config_line *line_handlers,
@@ -475,6 +483,7 @@ int
 snmp_config_when(char *line, int when)
 {
     char           *cptr, buf[STRINGMAX], tmpbuf[STRINGMAX];
+    char           *st;
     struct config_line *lptr = NULL;
     struct config_files *ctmp = config_files;
 
@@ -485,7 +494,7 @@ snmp_config_when(char *line, int when)
 
     strncpy(buf, line, STRINGMAX);
     buf[STRINGMAX - 1] = '\0';
-    cptr = strtok(buf, SNMP_CONFIG_DELIMETERS);
+    cptr = strtok_r(buf, SNMP_CONFIG_DELIMETERS, &st);
     if (cptr && cptr[0] == '[') {
         if (cptr[strlen(cptr) - 1] != ']') {
             snprintf(tmpbuf, sizeof(tmpbuf),
@@ -505,7 +514,7 @@ snmp_config_when(char *line, int when)
             config_perror(tmpbuf);
             return SNMPERR_GENERR;
         }
-        cptr = strtok(NULL, SNMP_CONFIG_DELIMETERS);
+        cptr = strtok_r(NULL, SNMP_CONFIG_DELIMETERS, &st);
         lptr = read_config_find_handler(lptr, cptr);
     } else {
         /*
@@ -523,7 +532,7 @@ snmp_config_when(char *line, int when)
     }
 
     /*
-     * use the original string instead since strtok messed up the original 
+     * use the original string instead since strtok_r messed up the original 
      */
     line = skip_white(line + (cptr - buf) + strlen(cptr) + 1);
 
@@ -760,14 +769,43 @@ free_config(void)
 }
 
 void
-read_configs(void)
+read_configs_optional(const char *optional_config, int when)
 {
-
-    char *optional_config = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
-					       NETSNMP_DS_LIB_OPTIONALCONFIG);
+    char *newp, *cp, *st;
     char *type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				       NETSNMP_DS_LIB_APPTYPE);
 
+    if ((NULL == optional_config) || (NULL == type))
+        return;
+
+    DEBUGMSGTL(("read_configs_optional",
+                "reading optional configuration tokens for %s\n", type));
+    
+    newp = strdup(optional_config);      /* strtok_r messes it up */
+    cp = strtok_r(newp, ",", &st);
+    while (cp) {
+        struct stat     statbuf;
+        if (stat(cp, &statbuf)) {
+            DEBUGMSGTL(("read_config",
+                        "Optional File \"%s\" does not exist.\n", cp));
+            snmp_log_perror(cp);
+        } else {
+            DEBUGMSGTL(("read_config",
+                        "Reading optional config file: \"%s\"\n", cp));
+            read_config_with_type_when(cp, type, when);
+        }
+        cp = strtok_r(NULL, ",", &st);
+    }
+    free(newp);
+    
+}
+
+void
+read_configs(void)
+{
+    char *optional_config = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
+					       NETSNMP_DS_LIB_OPTIONALCONFIG);
+
     DEBUGMSGTL(("read_config", "reading normal configuration tokens\n"));
 
     if (!netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
@@ -778,27 +816,8 @@ read_configs(void)
     /*
      * do this even when the normal above wasn't done 
      */
-    if (optional_config && type) {
-      char *newp, *cp;
-      newp = strdup(optional_config);
-      cp = strtok(newp, ",");
-      while (cp) {
-        struct stat     statbuf;
-        if (stat(cp, &statbuf)) {
-            DEBUGMSGTL(("read_config",
-                        "Optional File \"%s\" does not exist.\n",
-                        cp));
-            snmp_log_perror(cp);
-        } else {
-            DEBUGMSGTL(("read_config",
-                        "Reading optional config file: \"%s\"\n",
-                        cp));
-            read_config_with_type(cp, type);
-        }
-        cp = strtok(NULL, ",");
-      }
-      free(newp);
-    }
+    if (NULL != optional_config)
+        read_configs_optional(optional_config, NORMAL_CONFIG);
 
     netsnmp_config_process_memories_when(NORMAL_CONFIG, 1);
 
@@ -811,6 +830,9 @@ read_configs(void)
 void
 read_premib_configs(void)
 {
+    char *optional_config = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
+					       NETSNMP_DS_LIB_OPTIONALCONFIG);
+
     DEBUGMSGTL(("read_config", "reading premib configuration tokens\n"));
 
     if (!netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
@@ -818,6 +840,9 @@ read_premib_configs(void)
         read_config_files(PREMIB_CONFIG);
     }
 
+    if (NULL != optional_config)
+        read_configs_optional(optional_config, PREMIB_CONFIG);
+
     netsnmp_config_process_memories_when(PREMIB_CONFIG, 0);
 
     netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, 
@@ -1509,6 +1534,7 @@ read_config_read_octet_string(char *readfrom, u_char ** str, size_t * len)
             *len = strlen(readfrom);
 
         if (*len % 2) {
+            snmp_log(LOG_WARNING,"invalid hex string: wrong length\n");
             DEBUGMSGTL(("read_config_read_octet_string",
                         "invalid hex string: wrong length"));
             return NULL;
