@@ -397,7 +397,7 @@ netsnmp_tcp_create_ostring(const u_char * o, size_t o_len, int local)
         unsigned short porttmp = (o[4] << 8) + o[5];
         addr.sin_family = AF_INET;
         memcpy((u_char *) & (addr.sin_addr.s_addr), o, 4);
-        addr.sin_port = porttmp;
+        addr.sin_port = htons(porttmp);
         return netsnmp_tcp_transport(&addr, local);
     }
     return NULL;
