@@ -239,6 +239,7 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
      * response.  Linux turns the failed ICMP response into an error message
      * and return value, unlike all other OS's.  
      */
+    if (0 == netsnmp_os_prematch("Linux","2.4"))
     {
         int             one = 1;
         setsockopt(t->sock, SOL_SOCKET, SO_BSDCOMPAT, (void *) &one,
@@ -705,6 +706,10 @@ netsnmp_udp_getSecName(void *opaque, int olength,
     struct sockaddr_in *from = (struct sockaddr_in *) opaque;
     char           *ztcommunity = NULL;
 
+    if (secName != NULL) {
+        *secName = NULL;  /* Haven't found anything yet */
+    }
+
     /*
      * Special case if there are NO entries (as opposed to no MATCHING
      * entries).  
@@ -712,9 +717,6 @@ netsnmp_udp_getSecName(void *opaque, int olength,
 
     if (com2SecList == NULL) {
         DEBUGMSGTL(("netsnmp_udp_getSecName", "no com2sec entries\n"));
-        if (secName != NULL) {
-            *secName = NULL;
-        }
         return 0;
     }
 
@@ -727,9 +729,6 @@ netsnmp_udp_getSecName(void *opaque, int olength,
         from->sin_family != AF_INET) {
         DEBUGMSGTL(("netsnmp_udp_getSecName",
 		    "no IPv4 source address in PDU?\n"));
-        if (secName != NULL) {
-            *secName = NULL;
-        }
         return 1;
     }
 
@@ -789,7 +788,7 @@ netsnmp_udp_create_ostring(const u_char * o, size_t o_len, int local)
         unsigned short porttmp = (o[4] << 8) + o[5];
         addr.sin_family = AF_INET;
         memcpy((u_char *) & (addr.sin_addr.s_addr), o, 4);
-        addr.sin_port = porttmp;
+        addr.sin_port = htons(porttmp);
         return netsnmp_udp_transport(&addr, local);
     }
     return NULL;
