@@ -37,7 +37,6 @@
 #include <sys/socket.h>
 #endif
 
-#define HAVE_IF_NAMETOINDEX
 #if defined(HAVE_WINSOCK_H) || defined(cygwin)
     /*
      *  Windows IPv6 support is part of WinSock2 only
@@ -255,6 +254,7 @@ netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
      * response.  Linux turns the failed ICMP response into an error message
      * and return value, unlike all other OS's.  
      */
+    if (0 == netsnmp_os_prematch("Linux","2.4"))
     {
         int             one = 1;
         setsockopt(t->sock, SOL_SOCKET, SO_BSDCOMPAT, &one, sizeof(one));
@@ -790,6 +790,10 @@ inet_addrs_consistence(int pf, void *net, void *mask)
     switch (pf) {
     case PF_INET:
         tmp = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
+        if (!tmp) {
+            config_perror("Resource failure in inet_addr_consistence()");
+            return -1;
+        }
         memset(tmp, 0, sizeof(*tmp));
         tmp->sin_family = PF_INET;
         if (inet_addr_complement
@@ -799,6 +803,11 @@ inet_addrs_consistence(int pf, void *net, void *mask)
             return -1;
         }
         dst = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
+        if (!dst) {
+            config_perror("Resource failure in inet_addr_consistence()");
+            free(tmp);
+            return -1;
+        }
         memset(dst, 0, sizeof(*dst));
         dst->sin_family = PF_INET;
         if (inet_addr_and
@@ -815,6 +824,10 @@ inet_addrs_consistence(int pf, void *net, void *mask)
         break;
     case PF_INET6:
         tmp6 = (struct sockaddr_in6 *) malloc(sizeof(struct sockaddr_in6));
+        if (!tmp6) {
+            config_perror("Resource failure in inet_addr_consistence()");
+            return -1;
+        }
         memset(tmp6, 0, sizeof(*tmp6));
         tmp6->sin6_family = PF_INET6;
         if (inet_addr_complement
@@ -824,6 +837,11 @@ inet_addrs_consistence(int pf, void *net, void *mask)
             return -1;
         }
         dst6 = (struct sockaddr_in6 *) malloc(sizeof(struct sockaddr_in6));
+        if (!dst6) {
+            config_perror("Resource failure in inet_addr_consistence()");
+            free(tmp6);
+            return -1;
+        }
         memset(dst6, 0, sizeof(*dst6));
         dst6->sin6_family = PF_INET6;
         if (inet_addr_and
@@ -1212,6 +1230,10 @@ netsnmp_udp6_getSecName(void *opaque, int olength,
     char           *ztcommunity = NULL;
     char            str6[INET6_ADDRSTRLEN];
 
+    if (secName != NULL) {
+        *secName = NULL;  /* Haven't found anything yet */
+    }
+
     /*
      * Special case if there are NO entries (as opposed to no MATCHING
      * entries).  
@@ -1219,9 +1241,6 @@ netsnmp_udp6_getSecName(void *opaque, int olength,
 
     if (com2Sec6List == NULL) {
         DEBUGMSGTL(("netsnmp_udp6_getSecName", "no com2sec entries\n"));
-        if (secName != NULL) {
-            *secName = NULL;
-        }
         return 0;
     }
 
@@ -1234,9 +1253,6 @@ netsnmp_udp6_getSecName(void *opaque, int olength,
         || from->sin6_family != PF_INET6) {
         DEBUGMSGTL(("netsnmp_udp6_getSecName",
                     "no IPv6 source address in PDU?\n"));
-        if (secName != NULL) {
-            *secName = NULL;
-        }
         return 1;
     }
 
