@@ -90,11 +90,13 @@
  * logh_head:  A list of all log handlers, in increasing order of priority
  * logh_priorities:  'Indexes' into this list, by priority
  */
-netsnmp_log_handler *logh_head;
+netsnmp_log_handler *logh_head = NULL;
 netsnmp_log_handler *logh_priorities[LOG_DEBUG+1];
 
 static int      newline = 1;	 /* MTCRITICAL_RESOURCE */
 
+static char syslogname[64] = DEFAULT_LOG_ID;
+
 #ifndef HAVE_VSNPRINTF
                 /*
                  * Need to use the UCD-provided one 
@@ -281,7 +283,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
      * Finally, handle ".... -Lx value ...." syntax
      *   (*without* surrounding quotes)
      */
-    if (!*optarg) {
+    if ((!*optarg) && (NULL != argv)) {
         /*
          * We've run off the end of the argument
          *  so move on to the next.
@@ -371,8 +373,9 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
             int facility = decode_facility(optarg);
             if (facility == -1)  return -1;
             logh->pri_max = pri_max;
-            logh->token   = NULL;
+            logh->token   = strdup(snmp_log_syslogname(0));
             logh->magic   = (void *)facility;
+	    snmp_enable_syslog_ident(snmp_log_syslogname(0), facility);
 	}
         break;
 
@@ -383,6 +386,15 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
     return 0;
 }
 
+char *
+snmp_log_syslogname(const char *pstr)
+{
+  if (pstr)
+    strncpy (syslogname, pstr, sizeof(syslogname));
+
+  return syslogname;
+}
+
 void
 snmp_log_options_usage(const char *lead, FILE * outf)
 {
@@ -458,46 +470,27 @@ sprintf_stamp(time_t * now, char *sbuf)
     return sbuf;
 }
 
+void
+snmp_disable_syslog(void)
+{
+    netsnmp_log_handler *logh;
+
+    for (logh = logh_head; logh; logh = logh->next)
+        if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_SYSLOG) {
+
 #ifdef WIN32
-void
-snmp_disable_syslog(void)
-{
-    netsnmp_log_handler *logh;
-    HANDLE               eventlog_h;
-
-    for (logh = logh_head; logh; logh = logh->next)
-        if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_SYSLOG) {
-
-            eventlog_h = (HANDLE)logh->magic;
-            if (eventlog_h != NULL) {
-                if (!CloseEventLog(eventlog_h)) {
-	            /*
-	             * Hmmm.....
-	             * Maybe disable this handler, and log the error ?
-	             */
-                    fprintf(stderr, "Could not close event log. "
-                            "Last error: 0x%x\n", GetLastError());
-                } else {
-                    logh->magic = NULL;
-                }
+            if (logh->magic) {
+                HANDLE eventlog_h = (HANDLE)logh->magic;
+                CloseEventLog(eventlog_h);
+                logh->magic = NULL;
             }
-            logh->enabled = 0;
-	}
-}
 #else
-void
-snmp_disable_syslog(void)
-{
-    netsnmp_log_handler *logh;
-
-    for (logh = logh_head; logh; logh = logh->next)
-        if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_SYSLOG) {
             closelog();
             logh->imagic  = 0;
+#endif
             logh->enabled = 0;
 	}
 }
-#endif
 
 void
 snmp_disable_filelog(void)
@@ -515,6 +508,25 @@ snmp_disable_filelog(void)
 	}
 }
 
+/*
+ * returns that status of stderr logging
+ *
+ * @retval 0 : stderr logging disabled
+ * @retval 1 : stderr logging enabled
+ */
+snmp_stderrlog_status(void)
+{
+    netsnmp_log_handler *logh;
+
+    for (logh = logh_head; logh; logh = logh->next)
+        if (logh->enabled && (logh->type == NETSNMP_LOGHANDLER_STDOUT ||
+                              logh->type == NETSNMP_LOGHANDLER_STDERR)) {
+            return 1;
+	}
+
+    return 0;
+}
+
 void
 snmp_disable_stderrlog(void)
 {
@@ -552,7 +564,7 @@ snmp_disable_log(void)
 void
 snmp_enable_syslog(void)
 {
-    snmp_enable_syslog_ident(DEFAULT_LOG_ID, LOG_DAEMON);
+    snmp_enable_syslog_ident(snmp_log_syslogname(0), LOG_DAEMON);
 }
 
 void
@@ -580,9 +592,7 @@ snmp_enable_syslog_ident(const char *ident, const int facility)
         enable = 0;
     }
 #else
-    if (ident == NULL)
-        ident = DEFAULT_LOG_ID;
-    openlog(ident, LOG_CONS | LOG_PID, facility);
+    openlog(snmp_log_syslogname(ident), LOG_CONS | LOG_PID, facility);
 #endif
 
     for (logh = logh_head; logh; logh = logh->next)
@@ -637,7 +647,14 @@ snmp_enable_filelog(const char *logfilename, int dont_zero_log)
 {
     netsnmp_log_handler *logh;
 
-    snmp_disable_filelog();	/* XXX ??? */
+    /*
+     * don't disable ALL filelogs whenever a new one is enabled.
+     * this prevents '-Lf file' from working in snmpd, as the
+     * call to set up /var/log/snmpd.log will disable the previous
+     * log setup. again, this new linked list of log handlers
+     * needs rethinking/cleanup. xxx-rks
+     * snmp_disable_filelog();
+     */
 
     if (logfilename) {
         logh = netsnmp_find_loghandler( logfilename );
@@ -916,7 +933,8 @@ log_handler_syslog(  netsnmp_log_handler* logh, int pri, const char *string)
     }
     event_msg[0] = string;
     event_msg[1] = NULL;
-    if (!ReportEvent(eventlog_h, etype, 0, 0, NULL, 1, 0, event_msg, NULL)) {
+    /* NOTE: 4th parameter must match winservice.mc:MessageId value */
+    if (!ReportEvent(eventlog_h, etype, 0, 100, NULL, 1, 0, event_msg, NULL)) {
 	    /*
 	     * Hmmm.....
 	     * Maybe disable this handler, and log the error ?
@@ -1044,9 +1062,11 @@ snmp_log_string(int priority, const char *string)
     for ( ; logh; logh = logh->next ) {
         /*
          * ... but skipping any handlers with a "maximum priority"
-	 *     that we have already exceeded.
+         *     that we have already exceeded. And don't forget to
+         *     ensure this logging is turned on (see snmp_disable_stderrlog
+         *     and its cohorts).
          */
-        if (priority >= logh->pri_max)
+        if (logh->enabled && (priority >= logh->pri_max))
             logh->handler( logh, priority, string );
     }
 }
