@@ -208,6 +208,8 @@ netsnmp_daemonize(int quit_immediately, int stderr_log)
         }
 #ifndef WIN32
         else {
+            int cnt;
+
             /* Child. */
             
             DEBUGMSGT(("daemonize","child continuing\n"));
@@ -232,6 +234,12 @@ netsnmp_daemonize(int quit_immediately, int stderr_log)
                 dup(0);
                 dup(0);
             }
+
+            /*
+             * close all non-standard open file descriptors
+             */
+            for (cnt = getdtablesize() - 1; cnt >= 2; --cnt)
+                (void) close(cnt);
         }
 #endif /* !WIN32 */
     }
@@ -275,7 +283,7 @@ opendir(const char *filename)
     /*
      * check to see if filename is a directory 
      */
-    if (stat(filename, &sbuf) < 0 || sbuf.st_mode & S_IFDIR == 0) {
+    if ((stat(filename, &sbuf) < 0) || ((sbuf.st_mode & S_IFDIR) == 0)) {
         return NULL;
     }
 
@@ -502,7 +510,8 @@ get_uptime(void)
      * min requirement is one PERF_DATA_BLOCK plus one PERF_OBJECT_TYPE 
      */
     perfdata = (PPERF_DATA_BLOCK) malloc(buffersize);
-
+    if (!perfdata)
+        return 0;
 
     memset(perfdata, 0, buffersize);
 
@@ -887,6 +896,7 @@ setenv(const char *name, const char *value, int overwrite)
 }
 #endif                          /* HAVE_SETENV */
 
+/* returns centiseconds */
 int
 calculate_time_diff(struct timeval *now, struct timeval *then)
 {
@@ -903,6 +913,25 @@ calculate_time_diff(struct timeval *now, struct timeval *then)
     return ((diff.tv_sec * 100) + (diff.tv_usec / 10000));
 }
 
+/* returns diff in rounded seconds */
+u_int
+calculate_sectime_diff(struct timeval *now, struct timeval *then)
+{
+    struct timeval  tmp, diff;
+    memcpy(&tmp, now, sizeof(struct timeval));
+    tmp.tv_sec--;
+    tmp.tv_usec += 1000000L;
+    diff.tv_sec = tmp.tv_sec - then->tv_sec;
+    diff.tv_usec = tmp.tv_usec - then->tv_usec;
+    if (diff.tv_usec > 1000000L) {
+        diff.tv_usec -= 1000000L;
+        diff.tv_sec++;
+    }
+    if (diff.tv_usec >= 500000L)
+        return diff.tv_sec + 1;
+    return  diff.tv_sec;
+}
+
 #ifndef HAVE_STRCASESTR
 /*
  * only glibc2 has this.
@@ -965,6 +994,7 @@ mkdirhier(const char *pathname, mode_t mode, int skiplast)
     char           *ourcopy = strdup(pathname);
     char           *entry;
     char            buf[SNMP_MAXPATH];
+    char           *st;
 
 #if defined (WIN32) || defined (cygwin)
     /* convert backslash to forward slash */
@@ -973,7 +1003,7 @@ mkdirhier(const char *pathname, mode_t mode, int skiplast)
             *entry = '/';
 #endif
 
-    entry = strtok(ourcopy, "/");
+    entry = strtok_r(ourcopy, "/", &st);
 
     buf[0] = '\0';
 
@@ -985,7 +1015,7 @@ mkdirhier(const char *pathname, mode_t mode, int skiplast)
     if ((entry) && (':' == entry[1]) &&
         (('\0' == entry[2]) || ('/' == entry[2]))) {
         strcat(buf, entry);
-        entry = strtok(NULL, "/");
+        entry = strtok_r(NULL, "/", &st);
     }
 #endif
 
@@ -995,7 +1025,7 @@ mkdirhier(const char *pathname, mode_t mode, int skiplast)
     while (entry) {
         strcat(buf, "/");
         strcat(buf, entry);
-        entry = strtok(NULL, "/");
+        entry = strtok_r(NULL, "/", &st);
         if (entry == NULL && skiplast)
             break;
         if (stat(buf, &sbuf) < 0) {
@@ -1028,3 +1058,46 @@ mkdirhier(const char *pathname, mode_t mode, int skiplast)
     free(ourcopy);
     return SNMPERR_SUCCESS;
 }
+
+/*
+ * This function was created to differentiate actions
+ * that are appropriate for Linux 2.4 kernels, but not later kernels.
+ *
+ * This function can be used to test kernels on any platform that supports uname().
+ *
+ * If not running a platform that supports uname(), return -1.
+ *
+ * If ospname matches, and the release matches up through the prefix,
+ *  return 0.
+ * If the release is ordered higher, return 1.
+ * Be aware that "ordered higher" is not a guarantee of correctness.
+ */
+int
+netsnmp_os_prematch(const char *ospmname,
+                    const char *ospmrelprefix)
+{
+#if HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+static int printOSonce = 1;
+  struct utsname utsbuf;
+  if ( 0 != uname(&utsbuf))
+    return -1;
+
+  if (printOSonce) {
+    printOSonce = 0;
+    /* show the four elements that the kernel can be sure of */
+  DEBUGMSGT(("daemonize","sysname '%s',\nrelease '%s',\nversion '%s',\nmachine '%s'\n",
+      utsbuf.sysname, utsbuf.release, utsbuf.version, utsbuf.machine));
+  }
+  if (0 != strcasecmp(utsbuf.sysname, ospmname)) return -1;
+
+  /* Required to match only the leading characters */
+  return strncasecmp(utsbuf.release, ospmrelprefix, strlen(ospmrelprefix));
+
+#else
+
+  return -1;
+
+#endif /* HAVE_SYS_UTSNAME_H */
+}
+
