@@ -39,7 +39,7 @@ labelFIN: \
      * This should be declared by the application, which wants to register as
      * windows service
      */
-extern LPTSTR g_szAppName;
+extern LPTSTR app_name;
 
     /*
      * Declare global variable
@@ -85,15 +85,18 @@ static INT (*ServiceEntryPoint) (INT Argc, LPTSTR Argv[]) = 0L;
      */
 static VOID (*StopFunction) (VOID) = 0L;
 
+VOID
+ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int quiet);
+
     /*
      * To register as Windows Service with SCM(Service Control Manager)
      * Input - Service Name, Service Display Name,Service Description and
      * Service startup arguments
      */
-VOID
+int
 RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 		 LPCTSTR lpszServiceDescription,
-		 InputParams * StartUpArg) /* Startup argument to the service */
+		 InputParams * StartUpArg, int quiet) /* Startup argument to the service */
 {
   TCHAR szServicePath[MAX_PATH];	/* To hold module File name */
   TCHAR MsgErrorString[MAX_STR_SIZE];	/* Message or Error string */
@@ -107,6 +110,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
   HKEY hParamKey = NULL;	/* To store startup parameters */
   DWORD dwData;			/* Type of logging supported */
   DWORD i, j;			/* Loop variables */
+  int exitStatus = 0;
   GetModuleFileName (NULL, szServicePath, MAX_PATH);
   TRY
   {
@@ -117,14 +121,15 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
     hSCManager = OpenSCManager (NULL, NULL, SC_MANAGER_CREATE_SERVICE);
     if (hSCManager == NULL)
       {
-	DisplayError (_T ("Can't open SCM"));
+        ProcessError (EVENTLOG_ERROR_TYPE, _T ("Can't open SCM (Service Control Manager)"), 1, quiet);
+        exitStatus = SERVICE_ERROR_SCM_OPEN;
 	LEAVE;
       }
 
     /*
      * Generate the Command to be executed by SCM 
      */
-    _stprintf (szServiceCommand, "%s %s", szServicePath, _T ("-service"));
+    _snprintf (szServiceCommand, sizeof(szServiceCommand), "%s %s", szServicePath, _T ("-service"));
 
     /*
      * Create the Desired service 
@@ -139,9 +144,11 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 			      NULL);	/* password */
     if (hService == NULL)
       {
-	_stprintf (MsgErrorString, "%s %s",
-		   _T ("Can't Create Service"), lpszServiceDisplayName);
-	DisplayError (MsgErrorString);
+	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+		   _T ("Can't create service"), lpszServiceDisplayName);
+        ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
+
+        exitStatus = SERVICE_ERROR_CREATE_SERVICE;
 	LEAVE;
       }
 
@@ -159,9 +166,10 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
      */
     if (RegCreateKey (HKEY_LOCAL_MACHINE, szRegKey, &hKey) != ERROR_SUCCESS)
       {
-	_stprintf (MsgErrorString, "%s %s",
-		   _T ("Unable to create registry entries"), lpszServiceDisplayName);
-	DisplayError (MsgErrorString);
+	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+		   _T ("is unable to create registry entries"), lpszServiceDisplayName);
+        ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
+        exitStatus = SERVICE_ERROR_CREATE_REGISTRY_ENTRIES;
 	LEAVE;
       }
 
@@ -193,7 +201,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 	 * Create Registry Key path 
 	 */
 	_tcscpy (szRegKey, _T ("SYSTEM\\CurrentControlSet\\Services\\"));
-	_tcscat (szRegKey, g_szAppName);
+	_tcscat (szRegKey, app_name);
 	hKey = NULL;
 
 	/*
@@ -202,10 +210,11 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 	if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_WRITE,
 			  &hKey) != ERROR_SUCCESS)
 	  {
-	    _stprintf (MsgErrorString, "%s %s",
-		       _T ("Unable to create registry entries"),
+	    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+		       _T ("is unable to create registry entries"),
 		       lpszServiceDisplayName);
-	    DisplayError (MsgErrorString);
+            ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
+            exitStatus = SERVICE_ERROR_CREATE_REGISTRY_ENTRIES;
 	    LEAVE;
 	  }
 
@@ -219,10 +228,11 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 			       _tcslen (lpszServiceDescription) +
 			       sizeof (TCHAR)) != ERROR_SUCCESS)
 	      {
-		_stprintf (MsgErrorString, "%s %s",
-			   _T ("Unable to create registry entries"),
+		_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+			   _T ("is unable to create registry entries"),
 			   lpszServiceDisplayName);
-		DisplayError (MsgErrorString);
+                ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
+                exitStatus = SERVICE_ERROR_CREATE_REGISTRY_ENTRIES;
 		LEAVE;
 	      };
 	  }
@@ -240,11 +250,12 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 		 REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL,
 		 &hParamKey, NULL) != ERROR_SUCCESS)
 	      {
-		_stprintf (MsgErrorString, "%s %s",
-			   _T ("Unable to create registry entries"),
+		_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+			   _T ("is unable to create registry entries"),
 			   lpszServiceDisplayName);
-		DisplayError (MsgErrorString);
-		LEAVE;
+                ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
+                exitStatus = SERVICE_ERROR_CREATE_REGISTRY_ENTRIES;
+                LEAVE;
 	      }
 
 	    /*
@@ -254,9 +265,14 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 	    /*
 	     * Loop through arguments 
 	     */
-	    for (i = 2, j = 1; i < StartUpArg->Argc; i++, j++)
+            if (quiet) /* Make sure we don't store -quiet arg */
+              i = 3;
+            else
+              i = 2;
+
+	    for (j = 1; i < StartUpArg->Argc; i++, j++)
 	      {
-		_stprintf (szRegKey, "%s%d", _T ("Param"), j);
+		_snprintf (szRegKey, sizeof(szRegKey), "%s%d", _T ("Param"), j);
 
 		/*
 		 * Create registry key 
@@ -267,10 +283,11 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 		     _tcslen (StartUpArg->Argv[i]) +
 		     sizeof (TCHAR)) != ERROR_SUCCESS)
 		  {
-		    _stprintf (MsgErrorString, "%s %s",
-			       _T ("Unable to create registry entries"),
+		    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+			       _T ("is unable to create registry entries"),
 			       lpszServiceDisplayName);
-		    DisplayError (MsgErrorString);
+                    ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
+                    exitStatus = SERVICE_ERROR_CREATE_REGISTRY_ENTRIES;
 		    LEAVE;
 		  };
 	      }
@@ -290,13 +307,13 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
     /*
      * Successfully registered as service 
      */
-    _stprintf (MsgErrorString, "%s %s", lpszServiceName,
-	       _T ("- Successfully registered as Service"));
+    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", lpszServiceName,
+	       _T ("successfully registered as a service"));
 
     /*
      * Log message to eventlog 
      */
-    WriteToEventLog (EVENTLOG_INFORMATION_TYPE, MsgErrorString);
+    ProcessError (EVENTLOG_INFORMATION_TYPE, MsgErrorString, 0, quiet);
   }
 
   FINALLY
@@ -310,14 +327,15 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
     if (hParamKey)
       RegCloseKey (hParamKey);
   }
+  return (exitStatus);
 }
 
     /*
      * Unregister the service with the  Windows SCM 
      * Input - ServiceName
      */
-VOID
-UnregisterService (LPCSTR lpszServiceName)
+int
+UnregisterService (LPCSTR lpszServiceName, int quiet)
 {
   TCHAR MsgErrorString[MAX_STR_SIZE];	/* Message or Error string */
   SC_HANDLE hSCManager = NULL;	/* SCM handle */
@@ -326,6 +344,7 @@ UnregisterService (LPCSTR lpszServiceName)
   TCHAR szRegAppLogKey[] =
     "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\";
   TCHAR szRegKey[512];
+  int exitStatus = 0;
 /*  HKEY hKey = NULL;		?* Key to registry entry */
   TRY
   {
@@ -335,7 +354,8 @@ UnregisterService (LPCSTR lpszServiceName)
     hSCManager = OpenSCManager (NULL, NULL, SC_MANAGER_CREATE_SERVICE);
     if (hSCManager == NULL)
       {
-	MessageBox (NULL, _T ("Can't open SCM"), g_szAppName, MB_ICONHAND);
+        ProcessError (EVENTLOG_ERROR_TYPE, _T ("Can't open SCM (Service Control Manager)"), 1, quiet);
+        exitStatus = SERVICE_ERROR_SCM_OPEN;       
 	LEAVE;
       }
 
@@ -345,9 +365,10 @@ UnregisterService (LPCSTR lpszServiceName)
     hService = OpenService (hSCManager, lpszServiceName, SERVICE_ALL_ACCESS);
     if (hService == NULL)
       {
-	_stprintf (MsgErrorString, "%s %s", _T ("Can't open service"),
+	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", _T ("Can't open service"),
 		   lpszServiceName);
-	MessageBox (NULL, MsgErrorString, g_szAppName, MB_ICONHAND);
+        ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
+        exitStatus = SERVICE_ERROR_OPEN_SERVICE;       
 	LEAVE;
       }
 
@@ -369,21 +390,21 @@ UnregisterService (LPCSTR lpszServiceName)
      */
     if (DeleteService (hService) == FALSE)
       {
-	_stprintf (MsgErrorString, "%s %s", _T ("Can't delete service"),
+	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", _T ("Can't delete service"),
 		   lpszServiceName);
 
 	/*
 	 * Log message to eventlog 
 	 */
-	WriteToEventLog (EVENTLOG_INFORMATION_TYPE, MsgErrorString);
+        ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 0, quiet);
 	LEAVE;
       }
 
     /*
      * Log "Service deleted successfully " message to eventlog
      */
-    _stprintf (MsgErrorString, "%s %s", lpszServiceName, _T ("- Service deleted"));
-    WriteToEventLog (EVENTLOG_INFORMATION_TYPE, MsgErrorString);
+    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", lpszServiceName, _T ("service deleted"));
+    ProcessError (EVENTLOG_INFORMATION_TYPE, MsgErrorString, 0, quiet);
 
     /*
      * Delete registry entries for EventLog 
@@ -403,6 +424,7 @@ UnregisterService (LPCSTR lpszServiceName)
     if (hSCManager)
       CloseServiceHandle (hSCManager);
   }
+  return (exitStatus);
 }
 
     /*
@@ -417,23 +439,16 @@ WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
   va_list ArgList;
   HANDLE hEventSource = NULL;
   va_start (ArgList, pszFormat);
-  _vstprintf (szMessage, pszFormat, ArgList);
+  _vsnprintf (szMessage, sizeof(szMessage), pszFormat, ArgList);
   va_end (ArgList);
   LogStr[0] = szMessage;
-  hEventSource = RegisterEventSource (NULL, g_szAppName);
+  hEventSource = RegisterEventSource (NULL, app_name);
   if (hEventSource == NULL)
     return;
   ReportEvent (hEventSource, wType, 0,
 	       DISPLAY_MSG,	/* To Just output the text to event log */
 	       NULL, 1, 0, LogStr, NULL);
   DeregisterEventSource (hEventSource);
-  if (!g_fRunningAsService)
-    {
-      /*
-       * We are running in command mode, output the string 
-       */
-      _putts (szMessage);
-    }
 }
 
     /*
@@ -447,9 +462,10 @@ WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
      * Return: Type indicating the option specified
      */
 INT
-ParseCmdLineForServiceOption (int argc, TCHAR * argv[])
+ParseCmdLineForServiceOption (int argc, TCHAR * argv[], int *quiet)
 {
   int nReturn = RUN_AS_CONSOLE;	/* Defualted to run as console */
+
   if (argc >= 2)
     {
 
@@ -471,35 +487,111 @@ ParseCmdLineForServiceOption (int argc, TCHAR * argv[])
 	  nReturn = RUN_AS_SERVICE;
 	}
     }
+
+  if (argc >= 3)
+  {
+    /*
+     * third argument present 
+     */
+    if (lstrcmpi (_T ("-quiet"), argv[2]) == 0)
+    {
+      *quiet = 1;	
+    }
+  }
+  
   return nReturn;
 }
 
     /*
-     * To Display an error message describing the last system error
-     * message, along with a title passed as a parameter.
+     * Write error message to Event Log, console or pop-up window
+     *
+     * If useGetLastError is 1, the last error returned from GetLastError()
+     * is appended to pszMessage, separated by a ": ".
+     *
+     * eventLogType:                 MessageBox equivalent:
+     * 
+     * EVENTLOG_INFORMATION_TYPE     MB_ICONASTERISK
+     * EVENTLOG_WARNING_TYPE         MB_ICONEXCLAMATION
+     * EVENTLOG_ERROR_TYPE           MB_ICONSTOP
+     * 
      */
 VOID
-DisplayError (LPCTSTR pszTitle)
+ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int quiet)
 {
-  LPVOID pErrorMsg;
+  LPTSTR pErrorMsgTemp = NULL;
+  HANDLE hEventSource = NULL;
+  TCHAR pszMessageFull[MAX_STR_SIZE]; /* Combined pszMessage and GetLastError */
 
   /*
-   * Build Error String 
+   * If useGetLastError enabled, generate text from GetLastError() and append to
+   * pszMessageFull
    */
+  if (useGetLastError) {
   FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER |
 		 FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (),
 		 MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
-		 (LPTSTR) & pErrorMsg, 0, NULL);
-  if (g_fRunningAsService != FALSE)
-    {
-      WriteToEventLog (EVENTLOG_ERROR_TYPE, pErrorMsg);
+        (LPTSTR) & pErrorMsgTemp, 0, NULL);
+
+    _snprintf (pszMessageFull, sizeof(pszMessageFull), "%s: %s", pszMessage, pErrorMsgTemp);
+    if (pErrorMsgTemp) {
+      LocalFree (pErrorMsgTemp);
+      pErrorMsgTemp = NULL;
+    }
+  }
+  else {
+    _snprintf (pszMessageFull, sizeof(pszMessageFull), "%s", pszMessage);
+  }
+  
+  hEventSource = RegisterEventSource (NULL, app_name);
+  if (hEventSource != NULL) {
+    pErrorMsgTemp = pszMessageFull;
+    
+    if (ReportEvent (hEventSource, 
+          eventLogType, 
+          0,
+          DISPLAY_MSG,	/* To Just output the text to event log */
+          NULL, 
+          1, 
+          0, 
+          &pErrorMsgTemp, 
+          NULL)) {
+    }
+    else {
+      FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER |
+          FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (),
+          MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+          (LPTSTR) & pErrorMsgTemp, 0, NULL);
+      
+      fprintf(stderr,"Could NOT lot to Event Log.  Error returned from ReportEvent(): %s\n",pErrorMsgTemp);
+      if (pErrorMsgTemp) {
+        LocalFree (pErrorMsgTemp);
+        pErrorMsgTemp = NULL;
+      }
+    }
+    DeregisterEventSource (hEventSource);
     }
 
-  else
-    {
-      MessageBox (NULL, pErrorMsg, pszTitle, MB_ICONHAND);
+      if (quiet) {
+    fprintf(stderr,"%s\n",pszMessageFull);
+      }
+      else {
+    switch (eventLogType) {
+      case EVENTLOG_INFORMATION_TYPE:
+        MessageBox (NULL, pszMessageFull, app_name, MB_ICONASTERISK);
+        break;
+      case EVENTLOG_WARNING_TYPE:
+        MessageBox (NULL, pszMessageFull, app_name, MB_ICONEXCLAMATION);
+        break;
+      case EVENTLOG_ERROR_TYPE:
+        MessageBox (NULL, pszMessageFull, app_name, MB_ICONSTOP);
+        break;
+      default:
+        MessageBox (NULL, pszMessageFull, app_name, EVENTLOG_WARNING_TYPE);
+        break;
+      }
     }
-  LocalFree (pErrorMsg);
+  
+  LocalFree (pErrorMsgTemp);  
 }
 
     /*
@@ -590,8 +682,8 @@ ServiceMain (DWORD argc, LPTSTR argv[])
   /*
    * Create Registry Key path 
    */
-  _stprintf (szRegKey, "%s%s\\%s",
-	     _T ("SYSTEM\\CurrentControlSet\\Services\\"), g_szAppName,
+  _snprintf (szRegKey, sizeof(szRegKey), "%s%s\\%s",
+	     _T ("SYSTEM\\CurrentControlSet\\Services\\"), app_name,
 	     "Parameters");
   if (RegOpenKeyEx
       (HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_ALL_ACCESS, &hParamKey) == ERROR_SUCCESS)
@@ -615,6 +707,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 	       * Allocate memory to hold strings 
 	       */
 	      ArgArray = (LPTSTR *) malloc (sizeof (LPTSTR) * ArgCount);
+	      if (ArgArray == 0)
+                {
+                    WriteToEventLog (EVENTLOG_ERROR_TYPE,
+		       _T ("Resource failure"));
+                    return;
+                }
 
 	      /*
 	       * Copy first argument 
@@ -626,7 +724,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 		  /*
 		   * Create Subkey value name 
 		   */
-		  _stprintf (szRegKey, "%s%d", "Param", i);
+		  _snprintf (szRegKey, sizeof(szRegKey), "%s%d", "Param", i);
 
 		  /*
 		   * Set size 
@@ -659,7 +757,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
   /*
    * Register Service Control Handler 
    */
-  hServiceStatus = RegisterServiceCtrlHandler (g_szAppName, ControlHandler);
+  hServiceStatus = RegisterServiceCtrlHandler (app_name, ControlHandler);
   if (hServiceStatus == 0)
     {
       WriteToEventLog (EVENTLOG_ERROR_TYPE,
@@ -753,7 +851,7 @@ RunAsService (INT (*ServiceFunction) (INT, LPTSTR *))
   /*
    * Initialize ServiceTableEntry table 
    */
-  ServiceTableEntry[0].lpServiceName = g_szAppName;	/* Application Name */
+  ServiceTableEntry[0].lpServiceName = app_name;	/* Application Name */
 
   /*
    * Call SCM via StartServiceCtrlDispatcher to run as Service 
@@ -767,7 +865,7 @@ RunAsService (INT (*ServiceFunction) (INT, LPTSTR *))
        * Some other error has occurred. 
        */
       WriteToEventLog (EVENTLOG_ERROR_TYPE,
-		       _T ("Couldn't start service - %s"), g_szAppName);
+		       _T ("Couldn't start service - %s"), app_name);
     }
   return g_fRunningAsService;
 }
@@ -956,3 +1054,4 @@ ProcessServiceContinue (VOID)
 
 #endif /* WIN32 */
 
+
