@@ -85,11 +85,44 @@ export SNMP_PERSISTENT_FILE
 if [ "x$SNMP_FLAGS" = "x" ]; then
     SNMP_FLAGS="-d"
 fi
+if test -x /bin/netstat ; then
+    NETSTAT=/bin/netstat
+elif test -x /usr/bin/netstat ; then
+    NETSTAT=/usr/bin/netstat
+elif test -x /usr/sbin/netstat ; then
+    # e.g. Tru64 Unix
+    NETSTAT=/usr/sbin/netstat
+else
+    NETSTAT=""
+fi
 if [ "x$SNMP_SNMPD_PORT" = "x" ]; then
     SNMP_SNMPD_PORT="8765"
+    MAX_RETRIES=3
+    if test -x "$NETSTAT" ; then
+        if test -z "$RANDOM"; then
+            RANDOM=2
+        fi
+        while :
+        do
+            IN_USE=`$NETSTAT -a 2>/dev/null | grep "[\.:]$SNMP_SNMPD_PORT "`
+            if [ $? -eq 0 ]; then
+                #ECHO "Port $SNMP_SNMPD_PORT in use:"
+                #echo "->$IN_USE"
+                SNMP_SNMPD_PORT=`expr $SNMP_SNMPD_PORT + \( $RANDOM % 100 \)`
+            else
+                #echo "Using port $SNMP_SNMPD_PORT"
+                break
+            fi
+            MAX_RETRIES=`expr $MAX_RETRIES - 1`
+            if [ $MAX_RETRIES -eq 0 ]; then
+                echo "ERROR: Could not find available port."
+                exit 255
+            fi
+        done
+    fi
 fi
 if [ "x$SNMP_SNMPTRAPD_PORT" = "x" ]; then
-    SNMP_SNMPTRAPD_PORT="8764"
+    SNMP_SNMPTRAPD_PORT=`expr $SNMP_SNMPD_PORT - 1`
 fi
 if [ "x$SNMP_TRANSPORT_SPEC" = "x" ];then
 	SNMP_TRANSPORT_SPEC="udp"
